@@ -18,7 +18,7 @@ from dvc.scheme import Schemes
 from dvc.utils import format_link, tmp_fname
 from dvc.utils.stream import IterStream
 
-from .base import BaseRemoteTree
+from .base import BaseTree
 
 logger = logging.getLogger(__name__)
 FOLDER_MIME_TYPE = "application/vnd.google-apps.folder"
@@ -85,7 +85,7 @@ class GDriveURLInfo(CloudURLInfo):
         self._spath = re.sub("/{2,}", "/", self._spath.rstrip("/"))
 
 
-class GDriveRemoteTree(BaseRemoteTree):
+class GDriveTree(BaseTree):
     scheme = Schemes.GDRIVE
     PATH_CLS = GDriveURLInfo
     REQUIRES = {"pydrive2": "pydrive2"}
@@ -130,7 +130,7 @@ class GDriveRemoteTree(BaseRemoteTree):
         self._validate_config()
         self._gdrive_user_credentials_path = (
             tmp_fname(os.path.join(self.repo.tmp_dir, ""))
-            if os.getenv(GDriveRemoteTree.GDRIVE_CREDENTIALS_DATA)
+            if os.getenv(GDriveTree.GDRIVE_CREDENTIALS_DATA)
             else config.get(
                 "gdrive_user_credentials_file",
                 os.path.join(
@@ -172,8 +172,8 @@ class GDriveRemoteTree(BaseRemoteTree):
         Useful for tests, exception messages, etc. Returns either env variable
         name if it's set or actual path to the credentials file.
         """
-        if os.getenv(GDriveRemoteTree.GDRIVE_CREDENTIALS_DATA):
-            return GDriveRemoteTree.GDRIVE_CREDENTIALS_DATA
+        if os.getenv(GDriveTree.GDRIVE_CREDENTIALS_DATA):
+            return GDriveTree.GDRIVE_CREDENTIALS_DATA
         if os.path.exists(self._gdrive_user_credentials_path):
             return self._gdrive_user_credentials_path
         return None
@@ -187,7 +187,7 @@ class GDriveRemoteTree(BaseRemoteTree):
         DVC config client id or secret but forgets to remove the cached
         credentials file.
         """
-        if not os.getenv(GDriveRemoteTree.GDRIVE_CREDENTIALS_DATA):
+        if not os.getenv(GDriveTree.GDRIVE_CREDENTIALS_DATA):
             if (
                 settings["client_config"]["client_id"]
                 != auth.credentials.client_id
@@ -210,11 +210,9 @@ class GDriveRemoteTree(BaseRemoteTree):
         from pydrive2.auth import GoogleAuth
         from pydrive2.drive import GoogleDrive
 
-        if os.getenv(GDriveRemoteTree.GDRIVE_CREDENTIALS_DATA):
+        if os.getenv(GDriveTree.GDRIVE_CREDENTIALS_DATA):
             with open(self._gdrive_user_credentials_path, "w") as cred_file:
-                cred_file.write(
-                    os.getenv(GDriveRemoteTree.GDRIVE_CREDENTIALS_DATA)
-                )
+                cred_file.write(os.getenv(GDriveTree.GDRIVE_CREDENTIALS_DATA))
 
         auth_settings = {
             "client_config_backend": "settings",
@@ -260,7 +258,7 @@ class GDriveRemoteTree(BaseRemoteTree):
                 gauth.ServiceAuth()
             else:
                 gauth.CommandLineAuth()
-                GDriveRemoteTree._validate_credentials(gauth, auth_settings)
+                GDriveTree._validate_credentials(gauth, auth_settings)
 
         # Handle AuthenticationError, RefreshError and other auth failures
         # It's hard to come up with a narrow exception, since PyDrive throws
@@ -269,7 +267,7 @@ class GDriveRemoteTree(BaseRemoteTree):
         except Exception as exc:
             raise GDriveAuthError(self.credentials_location) from exc
         finally:
-            if os.getenv(GDriveRemoteTree.GDRIVE_CREDENTIALS_DATA):
+            if os.getenv(GDriveTree.GDRIVE_CREDENTIALS_DATA):
                 os.remove(self._gdrive_user_credentials_path)
 
         return GoogleDrive(gauth)
@@ -383,6 +381,9 @@ class GDriveRemoteTree(BaseRemoteTree):
         # it does not create a file on the remote
         gdrive_file = self._drive.CreateFile(param)
 
+        gdrive_file.FetchMetadata(fields="fileSize")
+        size = gdrive_file["fileSize"]
+
         with Tqdm(
             desc=progress_desc,
             disable=no_progress_bar,
@@ -390,7 +391,13 @@ class GDriveRemoteTree(BaseRemoteTree):
             # explicit `bar_format` as `total` will be set by `update_to`
             bar_format=Tqdm.BAR_FMT_DEFAULT,
         ) as pbar:
-            gdrive_file.GetContentFile(to_file, callback=pbar.update_to)
+            if size:
+                gdrive_file.GetContentFile(to_file, callback=pbar.update_to)
+            else:
+                # PyDrive doesn't like downloading empty files
+                # https://github.com/iterative/dvc/issues/4286
+                with open(to_file, "w"):
+                    pass
 
     @contextmanager
     @_gdrive_retry
@@ -524,7 +531,7 @@ class GDriveRemoteTree(BaseRemoteTree):
         assert not create
         raise FileMissingError(path_info, hint)
 
-    def exists(self, path_info):
+    def exists(self, path_info, use_dvcignore=True):
         try:
             self._get_item_id(path_info)
         except FileMissingError:
